package animation;

import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.JFrame;
import javax.swing.JTree;
import javax.swing.SwingUtilities;

import animation.PseudoCodeNode.CodeStatus;
import graph.LayeredGraphNode;

public abstract class AnimatedAlgorithm extends Thread implements AlgorithmStage {

    protected AnimationController ac;
    protected LayeredGraphNode graph;
    private JFrame view;
    protected PseudoCodeNode root;
    protected Memory mem;

    public AnimatedAlgorithm( AnimationController controller, LayeredGraphNode graph, JFrame view )
    {
        this.ac = controller;
        this.graph = graph;
        this.view = view;
        root = null;
        mem = new Memory();
        mem.declare( "graph", graph, true );
    }

    private void update()
    {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                view.repaint();
                for( ComponentListener l : view.getComponentListeners() )
                {
                    l.componentResized( new ComponentEvent(view, 0) );
                }
            }
        });
    }

    @Override
    public void run()
    {
        while( true ) // if this loop would end we could not undo steps any more
        {
            try {
                switch( ac.getNextAction() )
                {
                case FORWARD:
                    if( root.forwardStep( mem ) == CodeStatus.FINISHED )
                        ac.setContinuous( false );
                    update();
                    break;
                case FORWARD_OUT:
                    root.forwardStepOut( mem );
                    update();
                    break;
                case FORWARD_OVER:
                    root.forwardStepOver( mem );
                    update();
                    break;
                case BACKWARD:
                    if( root.backwardStep( mem ) == CodeStatus.FINISHED )
                        ac.setContinuous( false );
                    update();
                    break;
                case BACKWARD_OUT:
                    root.backwardStepOut( mem );
                    update();
                    break;
                case BACKWARD_OVER:
                    root.backwardStepOver( mem );
                    update();
                    break;
                default:
                    break;
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
                return;
            }

        }
    }

    @Override
    public abstract PseudoCodeNode createPseudocodeTree( JTree tree );
}
