package bk;

import javax.swing.JFrame;
import javax.swing.JTree;

import animation.AnimatedAlgorithm;
import animation.AnimationController;
import animation.CodeLine;
import animation.PseudoCodeNode;
import graph.LayeredGraphNode;
import lib.TextLayoutHelper;

/**
 * The main stage of the BK node placement algorithm.
 * @author kolja
 *
 */
public class BKNodePlacement extends AnimatedAlgorithm {

    /*
     * Private data structures to store the process of the algorithm
     */

    public enum State
    {
        CONFLICTS,
        LAYOUT1,
        LAYOUT2,
        LAYOUT3,
        LAYOUT4,
        COMBINE
    }

    private ConflictDetection conftion;
    private State state;
    private ExtremalLayoutCalc layouts[];
    private Combine combine;
    private PseudoCodeNode conflictsNode; 
    private PseudoCodeNode layout1Node;
    private PseudoCodeNode layout2Node;
    private PseudoCodeNode layout3Node;
    private PseudoCodeNode layout4Node;
    private PseudoCodeNode combineNode;
    private boolean inside;

    public BKNodePlacement(AnimationController controller, LayeredGraphNode graph, JFrame view) {
        super(controller, graph, view);
        state = State.CONFLICTS;
        conftion = new ConflictDetection( graph );
        layouts = new ExtremalLayoutCalc[ 4 ];
        layouts[ 0 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.TOP_BOTTOM_LEFT, graph );
        layouts[ 1 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.TOP_BOTTOM_RIGHT, graph );
        layouts[ 2 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.BOTTOM_TOP_LEFT, graph );
        layouts[ 3 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.BOTTOM_TOP_RIGHT, graph );
        combine = new Combine( graph );
        inside = false;
    }
    
    public State getAlgorithmState()
    {
        return state;
    }

    @Override
    public PseudoCodeNode createPseudocodeTree( JTree tree )
    {
        root = new PseudoCodeNode( TextLayoutHelper.setupPseudoCodeStage("BK Node Placement Algorithm"), tree, CodeLine.DEFAULT_STEP_INTO_STACK );
        root.setSelected( true );
        conflictsNode = conftion.createPseudocodeTree( tree );
        layout1Node = layouts[ 0 ].createPseudocodeTree( tree );
        layout2Node = layouts[ 1 ].createPseudocodeTree( tree );
        layout3Node = layouts[ 2 ].createPseudocodeTree( tree );
        layout4Node = layouts[ 3 ].createPseudocodeTree( tree );
        combineNode = combine.createPseudocodeTree( tree );
        root.add( conflictsNode );
        root.add( layout1Node );
        root.add( layout2Node );
        root.add( layout3Node );
        root.add( layout4Node );
        root.add( combineNode );
        return root;
    }

    @Override
    public String getDebugString()
    {
        if( !inside )
            return "";
        switch( state )
        {
        case CONFLICTS:
            return conftion.getDebugString();
        case LAYOUT1:
            return layouts[ 0 ].getDebugString();
        case LAYOUT2:
            return layouts[ 1 ].getDebugString();
        case LAYOUT3:
            return layouts[ 2 ].getDebugString();
        case LAYOUT4:
            return layouts[ 3 ].getDebugString();
        case COMBINE:
            return combine.getDebugString();
        }
        return "";
    }
}
